
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

        INTEGER FUNCTION GETMENU( ITEMCNT, DEFAULT, PROMPT, CHOICES )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: GETMENU
!DEC$ ENDIF
C...............................................................
C       function body starts at line  82
C
C       CALLS:      TRIMLEN
C       RETURNS:    user-selected item number from menu choices.
C
C  FUNCTION:
C
C       Display a menu to the screen; then display a prompt, get the
C       user's response and check that it is within range.
C       NOTE:  prompt and menu choice strings  should have
C       length <= 72 characters.  Ideally, number of items should be <= 18;
C       should be <= 999 and >= 1 in any case.
C
C  ARGUMENT LIST DESCRIPTION:
C
C    Input arguments:
C
C       ITEMCNT     number of items in menu. Ideally, < 20
C       DEFAULT     default menu choice
C       PROMPT      menu prompt
C       CHOICES     array of menu choices
C
C    Output arguments:  none
C
C  REVISION HISTORY:
C
C       Created  9/89 by CJC.
C       Modified 6/90 by CJC for ROM 2.2 -- error exit via EXWST
C       Modified 8/90 by CJC for ROM 2.2 -- TEMP.LET; exits on CONTROL-Z.
C       Modified 2/93 by CJC for CRAY
C       Modified 8/96 by CJC to treat "!" as a terminator
C       Modified 1/97 by CJC:  logs result
C
C...............................................................

        IMPLICIT NONE

C.......   ARGUMENTS:

        INTEGER         ITEMCNT         !  number of choices
        INTEGER         DEFAULT         !  default response
        CHARACTER*(*)   PROMPT          !  prompt string
        CHARACTER*(*)   CHOICES ( * )   !  array of choice strings


C.......   EXTERNAL FUNCTION:

        LOGICAL         ENVYN
        INTEGER         TRIMLEN
        EXTERNAL        ENVYN, TRIMLEN


C.......   LOCAL VARIABLES:

        INTEGER         LENGTH          !  length of default as ASCII string
        INTEGER         I , M, N
        INTEGER         ANSWER
        INTEGER         ERRCNT
        INTEGER         IOS
        CHARACTER*32    BUFFER
        CHARACTER*8     FMTSTR

        LOGICAL         PROMPTON

        LOGICAL         FIRSTIME
        DATA            FIRSTIME / .TRUE. /
        SAVE            FIRSTIME, PROMPTON

C...............................................................
C   begin body of GETMENU

        IF( FIRSTIME ) THEN

            PROMPTON = ENVYN( 'PROMPTFLAG', 'Prompt for input flag',
     &                      .TRUE., IOS )
            FIRSTIME = .FALSE.
 
        END IF

        IF( .NOT. PROMPTON ) THEN
            I =   TRIMLEN( CHOICES( DEFAULT ) )
            M  =  TRIMLEN( PROMPT )
            GETMENU = DEFAULT
            WRITE( *,92300 ) 
     &          'Using default response', CHOICES( DEFAULT )( 1:I ),
     &          ' for query', PROMPT( 1:M )
            RETURN
        END IF

        IF ( ITEMCNT .GT. 999 ) THEN
            WRITE ( *,91010 )
     &              'Maximum number of items:  ', 999,
     &              'Number of requested items:', ITEMCNT
            CALL M3EXIT( 'GETMENU', 0, 0, 
     &                   'Maximum item count exceeded.', 1 )
        ELSE IF ( ITEMCNT .LT. 1 ) THEN
            WRITE ( *,91010 )
     &              'Minimum number of items:  ', 1,
     &              'Number of requested items:', ITEMCNT
            CALL M3EXIT( 'GETMENU', 0, 0,
     &                   'Minimum item count exceeded.', 1 )
        END IF

        I       =  1
        LENGTH  =  0
10      CONTINUE

            I       =  I * 10
            LENGTH  =  LENGTH  +  1
            IF ( I .LE. DEFAULT )  GO TO  10

        ERRCNT =  0

100     CONTINUE

        WRITE ( *,* )
        WRITE ( *,92010 )
        IF ( ITEMCNT .LE. 18 )  WRITE ( *,92020 )

        DO  200  I = 1 , ITEMCNT

            M  =  TRIMLEN( CHOICES ( I ) )

            WRITE ( *,92030 ) I , CHOICES ( I ) ( 1:M )

200     CONTINUE

        M  =  TRIMLEN( PROMPT )

        IF ( ITEMCNT .LE. 18 )  WRITE ( *,92020 )
        WRITE ( *,92010 )
        IF ( ITEMCNT .LT. 10 )THEN
            WRITE ( *,95011 ) PROMPT ( 1:M ) , DEFAULT
        ELSE
            WRITE ( *,95012 ) PROMPT ( 1:M ) , DEFAULT
        END IF

        BUFFER =  '           '
        READ ( *, 92100, ERR=900, IOSTAT=IOS, END = 999 ) BUFFER

        N = INDEX( BUFFER, '!' )
        IF ( N .GT. 0 ) BUFFER( N : LEN( BUFFER ) ) = ' '

        IF ( BUFFER ( 1:1 )  .EQ. ' ' )  THEN
            GETMENU  =  DEFAULT
            WRITE( *,92200 ) 
     &              'Using default', DEFAULT, 
     &              CHOICES( DEFAULT )( 1:TRIMLEN( CHOICES( DEFAULT )))
        ELSE
            WRITE( FMTSTR, 94010 ) TRIMLEN( BUFFER )

            READ( BUFFER, FMTSTR, IOSTAT=IOS, ERR=400 ) ANSWER

            IF ( ANSWER .LT. 1  .OR.  ANSWER .GT. ITEMCNT )  THEN

                ERRCNT  =  ERRCNT + 1
                WRITE ( *,91020 ) ANSWER , ITEMCNT , ERRCNT
                IF ( ERRCNT .LT. 5 )  GO TO  100

                GO TO  500      !  max error count exceeded

            END IF

            GETMENU  =  ANSWER
            WRITE( *,92200 ) 
     &              'Using response', ANSWER, 
     &              CHOICES( ANSWER )( 1:TRIMLEN( CHOICES( ANSWER )))

        END IF

        RETURN


400     CONTINUE        !  error in read from BUFFER

        ERRCNT  =  ERRCNT + 1
        WRITE ( *,91030 )  ERRCNT
        IF ( ERRCNT .LT. 5 )  GO TO  100


500     CONTINUE        !  more than 5 entry errors

        CALL M3EXIT( 'GETMENU', 0, 0, 
     &               'Maximum error count exceeded', 2 )


900     CONTINUE        !  error in read from terminal

        IF ( IOS .GT. 0 ) THEN
            ERRCNT  =  ERRCNT + 1
            WRITE ( *,91030 )  ERRCNT
            IF ( ERRCNT .LT. 5 )  GO TO  100
            WRITE ( *,91040 ) IOS
            CALL M3EXIT( 'GETMENU', 0, 0, 'Error reading response', 2 )
        END IF
        
999     CONTINUE        !  ios negative, or ENDFILE condition

        CALL M3EXIT( 'GETMENU', 0, 0, 'END-OF-FILE encountered', 1 )


C.................   end body of GETMENU  ..............................

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( 2 ( /10X , A, I12 ) )

91020   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'      ,
     &         //10X , 'Your response'          , I6 , 2X,
     &                 'not in the range 1 ...' , I6 ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'            , / )

91030   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'         ,
     &         //10X , 'Did not understand your response' ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'                , / )

91040   FORMAT ( //10X  , 'Error number = ' , I3 )


C...........   Informational (LOG) message formats... 92xxx

92010   FORMAT ( 1X , 79('*') )

92020   FORMAT ( 1X , '*' , T79 , '*' )

92030   FORMAT ( 1X , '*  ' , I2 , ':  ' , A , T79 , '*' )

92100   FORMAT ( A )

92200   FORMAT ( /5X, A, I3, ' : "', A, '"' )

92300   FORMAT ( /5X, A, '"', A, '"', 1X, A, /5X,  A, '"', A, '"' )


C...........   Internal buffering formats............ 94xxx

94010   FORMAT( '( I', I3, ')' )


C...........   Miscellaneous formats   .............. 95xxx

95011   FORMAT (/5X , A , ' [' , I1 , '] >> ' , $ )

95012   FORMAT ( 5X , A , ' [' , I2 , '] >> ' , $ )


        END

C................   end   GETMENU  ....................................

