
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION  OPNFIL3 ( EQNAME, FID, FSTATUS, PGNAME )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: OPNFIL3
!DEC$ ENDIF
C***********************************************************************
C  function body starts at line  69
C
C  FUNCTION:  open pre-existing Models-3 file with logical name FLIST3( FID ),
C             with readonly/readwrite status FSTATUS.  If opened for
C             write, copy scenario description from I/O STATE3.EXT to
C             file's history, and name PGNAME of caller to file's
C             updater-name.  Returns TRUE if the file is already open.
C
C  RETURN VALUE:  TRUE iff it succeeds in opening the file, reading its
C                 attributes, and storing the relevant ones in STATE3.EXT
C
C  PRECONDITIONS REQUIRED:  File FLIST3( FID ) already exists.
C
C  SUBROUTINES AND FUNCTIONS CALLED:  TRIMLEN
C
C  REVISION  HISTORY:
C       prototype 9/94 by CJC to go with new version of OPEN3()
C
C	Modified  2/95 by CJC to handle file type SMATRX3.
C
C	Modified 10/95 by CJC to handle file type TSRIES3.
C
C	Modified 1/97 by CJC to handle file type PTRFLY3.
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   EQNAME  !  physical name of file to be opened
        INTEGER         FID     !  index into STATE3 tables
        INTEGER         FSTATUS !  read-only/read-write status for NCOPN()
        CHARACTER*16    PGNAME  !  name of calling program


C...........   EXTERNAL FUNCTIONS and their descriptions:

        LOGICAL         CKFILE3 !  header-consistency check
        INTEGER         TRIMLEN !  string length (except trailing blanks)
        EXTERNAL        CKFILE3, TRIMLEN


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER              FNUM       !  netCDF file ID from NCCRE()
        INTEGER              TDIM       !  netCDF dimension ID from NCDID
        INTEGER              IERR       !  netCDF error status return
        INTEGER              VAR        !  loop counter for file variables
        INTEGER              ND, NA
        INTEGER              D( MAXVDIMS )
        CHARACTER*(MAXNCNAM) TNAME      !  dummy arg for NCDINQ
        CHARACTER*256        EQ256      !  fixed-length buffer
        CHARACTER*256        MESG       !  fixed-length buffer

C.............................................................................
C   begin body of subroutine  OPNFIL3

        IF ( EQNAME( 1:8 ) .EQ. 'BUFFERED' ) THEN
            MESG = 'Error opening "BUFFERED" file ' // FLIST3( FID )
            CALL M3MSG2( MESG )
            CALL M3MSG2( 'File has not yet been created.' )
            RETURN
        END IF

C...........  Open the netCDF file with that EQNAME.

        EQ256= EQNAME    
        IERR = NF_OPEN( EQNAME, IOR( NF_SHARE, FSTATUS ), FNUM )
        IF ( IERR .NE. 0 ) THEN
            CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &      'Error opening file at path-name:' )
            CALL M3MSG2( EQ256 )
            OPNFIL3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCCRE() failed
        CDFID3( FID ) = FNUM


C...........   Get attributes valid for all file types:
C...........   FTYPE:  file type ID

        CALL NCAGT( FNUM, NCGLOBAL, 'FTYPE', FTYPE3( FID ), IERR )
        IF ( IERR .NE. 0 ) THEN
            CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &      'Error reading netCDF file attribute FTYPE.' )
            OPNFIL3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

C.......   NVARS:  number of variables

        CALL NCAGT( FNUM, NCGLOBAL, 'NVARS', NVARS3( FID ), IERR )
        IF ( IERR .NE. 0 ) THEN
            CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &      'Error reading netCDF file attribute NVARS' )
            OPNFIL3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        IF ( FTYPE3( FID ) .EQ. DCTNRY3 ) THEN

C...........   Set up variable FLAG:  "record-written" indicator, valid
C...........   for dictionary files:

            TINDX3( FID ) = NCVID( FNUM, 'FLAG', IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute FLAG' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

        ELSE    ! Set up attributes and variables valid for nondictionary files:

C.......   TSTEP:  time step

            IF ( FTYPE3( FID ) .NE. KFEVNT3 ) THEN
                CALL NCAGT( FNUM, NCGLOBAL,  
     &                      'TSTEP', TSTEP3( FID ), IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading file attribute TSTEP' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed
            END IF          !  if not kfevnt3

C.......   SDATE:  starting date (Julian date YYYYDDD)

            CALL NCAGT( FNUM, NCGLOBAL, 'SDATE', SDATE3( FID ) , IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute SDATE' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   STIME:  starting time (HHMMSS)

            CALL NCAGT( FNUM, NCGLOBAL, 'STIME', STIME3( FID ) , IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute STIME' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   NTHIK:  boundary file perimeter thickness (cells)

            CALL NCAGT( FNUM, NCGLOBAL, 'NTHIK', NTHIK3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &         'Error reading file attribute NTHIK' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   NCOLS:  number of grid columns/profile levels

            CALL NCAGT( FNUM, NCGLOBAL, 'NCOLS', NCOLS3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute NCOLS' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   NROWS:  number of grid rows/data sites

            CALL NCAGT( FNUM, NCGLOBAL, 'NROWS', NROWS3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute NROWS' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   NLAYS:  number of layers

            CALL NCAGT( FNUM, NCGLOBAL, 'NLAYS', NLAYS3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute NLAYS' )
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   GDTYP:  grid type ID (lat-lon, UTM, RADM, etc...)

            CALL NCAGT( FNUM, NCGLOBAL, 'GDTYP', GDTYP3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute GDTYP' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed


C.......   P_ALP:  first map-projection-description angle               

            CALL NCAGT( FNUM, NCGLOBAL, 'P_ALP', P_ALP3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute P_ALP' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   P_BET:  second map-projection-description angle              

            CALL NCAGT( FNUM, NCGLOBAL, 'P_BET', P_BET3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute P_BET' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   P_GAM:  third map-projection-description angle               

            CALL NCAGT( FNUM, NCGLOBAL, 'P_GAM', P_GAM3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute P_GAM' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   XCENT:  lon of coordinate-system (0,0) origin

            CALL NCAGT( FNUM, NCGLOBAL, 'XCENT', XCENT3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute XCENT' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   YCENT:  lat of coordinate-system (0,0) origin

            CALL NCAGT( FNUM, NCGLOBAL, 'YCENT', YCENT3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute YCENT' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   XORIG:  X-coord of grid origin
C.......   (in map units; see FDESC3.EXT for description)

            CALL NCAGT( FNUM, NCGLOBAL, 'XORIG', XORIG3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute XORIG' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   YORIG:  Y-coord of grid origin
C.......   (in map units; see FDESC3.EXT for description)

            CALL NCAGT( FNUM, NCGLOBAL, 'YORIG', YORIG3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute YORIG' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   XCELL:  cell width (X direction)
C.......   (in map units; see FDESC3.EXT for description)

            CALL NCAGT( FNUM, NCGLOBAL, 'XCELL', XCELL3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute XCELL' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   YCELL:  cell width (Y direction)
C.......   (in map units; see FDESC3.EXT for description)

            CALL NCAGT( FNUM, NCGLOBAL, 'YCELL', YCELL3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute XCELL' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C.......   GDNAM:  grid name

            CALL NCAGTC( FNUM, NCGLOBAL, 'GDNAM',
     &                   GDNAM3( FID ), NAMLEN3, IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute GDNAM' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed


C...........   Get ID for timestep-available-flag variable:

            TINDX3( FID ) = NCVID( FNUM, 'TFLAG', IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading file attribute TFLAG' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C...........   Get extent of the time dimension:

            TDIM = NCDID( FNUM, 'TSTEP', IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading time-dimension ID' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

            CALL NCDINQ( FNUM, TDIM, TNAME, MXREC3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &          'Error reading maximum timestep number' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

C...........   Now, get netCDF variable ID's

            IF ( FTYPE3( FID ) .EQ. CUSTOM3 )  THEN

                BSIZE3( FID ) = NCOLS3( FID )

            ELSE IF ( FTYPE3( FID ) .EQ. GRDDED3 )  THEN

                BSIZE3( FID ) = NCOLS3( FID ) * NROWS3( FID )

            ELSE IF ( FTYPE3( FID ) .EQ. BNDARY3 ) THEN

                BSIZE3( FID ) = 2 * ABS( NTHIK3( FID ) )
     &                            * ( NCOLS3( FID ) + NROWS3( FID ) 
     &                              + 2 * NTHIK3( FID ) )

            ELSE IF ( FTYPE3( FID ) .EQ. IDDATA3 ) THEN

                BSIZE3( FID ) = NROWS3( FID )

                NINDX3( FID ) = NCVID( FNUM, 'ID-COUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable ID-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                SINDX3( FID ) = NCVID( FNUM, 'ID', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netcdf ID for variable ID' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

            ELSE IF ( FTYPE3( FID ) .EQ. PROFIL3 ) THEN

                BSIZE3( FID ) = NCOLS3( FID ) * NROWS3( FID )

                NINDX3( FID ) = NCVID( FNUM, 'ID-COUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netcdf ID for variable ID-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                SINDX3( FID ) = NCVID( FNUM, 'ID', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netcdf ID for variable ID' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                LINDX3( FID ) = NCVID( FNUM, 'LEVEL-COUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable LEVEL-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                XINDX3( FID ) = NCVID( FNUM, 'X', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable X' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                YINDX3( FID ) = NCVID( FNUM, 'Y', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable Y' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                ZINDX3( FID ) = NCVID( FNUM, 'Z', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable Z' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

            ELSE IF ( FTYPE3( FID ) .EQ. GRNEST3 ) THEN

                BSIZE3( FID ) = NCOLS3( FID ) * NROWS3( FID )

                NINDX3( FID ) = NCVID( FNUM, 'ID-COUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable ID-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                SINDX3( FID ) = NCVID( FNUM, 'ID', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netcdf ID for variable "ID"' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                WCNDX3( FID ) = NCVID( FNUM, 'WIN-COLS', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable WIN-COLS' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                WRNDX3( FID ) = NCVID( FNUM, 'WIN-ROWS', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable WIN-ROWST' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                LINDX3( FID ) = NCVID( FNUM, 'WIN-LEVS', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable WIN-LEVS' )
                    MESG = 'File name:  ' // FLIST3( FID )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                XINDX3( FID ) = NCVID( FNUM, 'X', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable X' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                YINDX3( FID ) = NCVID( FNUM, 'Y', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable Y' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                DXNDX3( FID ) = NCVID( FNUM, 'DX', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable DX' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                DYNDX3( FID ) = NCVID( FNUM, 'DY', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable DY' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

            ELSE IF ( FTYPE3( FID ) .EQ. SMATRX3 ) THEN

                BSIZE3( FID ) = NCOLS3( FID )

                SINDX3( FID ) = NCVID( FNUM, 'COL-COUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable COL-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                LINDX3( FID ) = NCVID( FNUM, 'COEFF-INDEX', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading ID for variable COEFF-INDEX' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

            ELSE IF ( FTYPE3( FID ) .EQ. TSRIES3 )  THEN
 
                BSIZE3( FID ) = NCOLS3( FID ) * NROWS3( FID )

            ELSE IF ( FTYPE3( FID ) .EQ. PTRFLY3 )  THEN
 
                BSIZE3( FID ) = NCOLS3( FID ) * NROWS3( FID )

            ELSE IF ( FTYPE3( FID ) .EQ. KFEVNT3 ) THEN

                BSIZE3( FID ) = NROWS3( FID )

                NINDX3( FID ) = NCVID( FNUM, 'KFCOUNT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netCDF ID for variable ID-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF              !  ierr nonzero:  NCVID() failed

                SINDX3( FID ) = NCVID( FNUM, 'KFEVENT', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netCDF variable ID' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF              !  ierr nonzero:  NCVID() failed

                LINDX3( FID ) = NCVID( FNUM, 'KFSDATE', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netCDF ID for variable LEVEL-COUNT' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF              !  ierr nonzero:  NCVID() failed

                XINDX3( FID ) = NCVID( FNUM, 'KFSTIME', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netCDF ID for variable KFSDATE' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF              !  ierr nonzero:  NCVID() failed

                YINDX3( FID ) = NCVID( FNUM, 'KFLNGTH', IERR )
                IF ( IERR .NE. 0 ) THEN
                    CALL M3ABORT( FLIST3( FID ), FNUM, IERR, 
     &              'Error reading netCDF ID for variable KFLNGTH' )
                    OPNFIL3 = .FALSE.
                    RETURN
                END IF              !  ierr nonzero:  NCVID() failed

            ELSE    !  illegal file type

                MESG = 'File opening error:  ' // 
     &                 'ILLEGAL FILE TYPE for file ' // FLIST3( FID ) 
                    CALL M3MSG2( MESG )
                WRITE( MESG,91010 ) 'Unknown file type ', FTYPE3( FID )
                    CALL M3MSG2( MESG )
                OPNFIL3 = .FALSE.
                RETURN

            END IF      !  if file type custom, gridded, idlist, profile,
                        !  gridnest, smatrix, or none of the above.

        END IF          !  file type not dictionary


C.......   Variables-list for the file:

        IF ( NVARS3( FID ) .GT. 0 ) THEN
            CALL NCAGTC( FNUM, NCGLOBAL, 'VAR-LIST', VLIST3( 1,FID ),
     &                   NAMLEN3 * NVARS3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR,
     &          'Error reading attribute VAR-LIST' )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed
        END IF              !  if nvars > 0

        DO 100  VAR = 1 , NVARS3( FID )

            VINDX3( VAR,FID ) = NCVID( FNUM, VLIST3( VAR,FID ), IERR )
            IF ( IERR .NE. 0 ) THEN
                MESG = 'Error reading IDs for  variable '
     &          // VLIST3( VAR,FID )
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, MESG )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

            CALL NCVINQ( FNUM, 
     &                   VINDX3( VAR,FID ), TNAME, 
     &                   VTYPE3( VAR,FID ), ND, D, NA, IERR )
            IF ( IERR .NE. 0 ) THEN
                MESG = 'Error reading type for  variable '
     &          // VLIST3( VAR,FID )
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR, MESG )
                OPNFIL3 = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

100     CONTINUE


C.......   If opened for WRITE:  put attribute HISTORY:  update description

        IF ( IAND( FSTATUS, NF_WRITE ) .NE. 0 ) THEN

            CALL NCREDF( FNUM, IERR )
            IF ( IERR .NE. 0 ) THEN
               CALL M3ABORT( FLIST3( FID ), FNUM, IERR,
     &         'Error opening history for update' )
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

            CALL NCAPTC( FNUM, NCGLOBAL, 'UPNAM', NCCHAR,
     &                   NAMLEN3, PGNAME, IERR )
            IF ( IERR .NE. 0 ) THEN
               CALL M3ABORT( FLIST3( FID ), FNUM, IERR,
     &         'Error writing file attribute UPNAM' )
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

            CALL NCAPTC( FNUM, NCGLOBAL, 'HISTORY', NCCHAR,
     &                   MXDLEN3 * MXDESC3, SCNDSC, IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR,
     &          'Error writing file attribute HISTORY' )
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

            CALL NCENDF( FNUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                CALL M3ABORT( FLIST3( FID ), FNUM, IERR,
     &          'Error flushing HISTORY to disk' )
                RETURN
            END IF          !  ierr nonzero:  NCAGT() failed

        END IF
        
        IF ( .NOT. CKFILE3( FID ) ) THEN
            CALL NCCLOS( FNUM, IERR )
            OPNFIL3 = .FALSE.
        ELSE
            OPNFIL3 = .TRUE.
        END IF

        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( A , : I5 )

        END

