
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

      LOGICAL FUNCTION READ3(FNAME, VNAME, LAYER, JDATE, JTIME, BUFFER)

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: READ3
!DEC$ ENDIF
C***********************************************************************
C  function body starts at line  95
C
C  FUNCTION:  reads data from Models-3 data file with logical name FNAME
C             for variable with name VNAME and layer LAYER, for the
C             date and time JDATE (coded YYYYDDD) and time JTIME (HHMMSS).
C             For time-independent files, JDATE:JTIME are ignored.
C             If VNAME is 'ALL', reads all variables; if LAYER is -1,
C             reads all layers.
C             If FNAME is a dictionary file, READ3() treats VNAME as a
C             dictionary index, and returns  RDDICT3( FNAME, VNAMEE ).
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  FNAME is a Models-3 data file already opened
C                 by OPEN3()
C
C  REVISION  HISTORY:  
C       prototype 3/92 by CJC
C
C       Modified  7/94 by CJC to handle restart files (tstep < 0)
C
C       Modified  8/94 by CJC to handle BUFFERED "files", and pass
C       VID argument to new RD*() routines (instead of VNAME)
C
C       Modified 10/94 by CJC to handle new files with write granularity
C       at the level of individual variables.
C
C       Modified  2/95 by CJC to handle files of type SMATRX3
C
C       Modified  2/95 by CJC to handle files of type TSRIES3
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME           !  logical file name
        CHARACTER*(*)   VNAME           !  variable name, or 'ALL'
        INTEGER         LAYER           !  layer number, or 0
        INTEGER         JDATE           !  date, formatted YYYYDDD
        INTEGER         JTIME           !  time, formatted HHMMSS
        REAL            BUFFER(*)       !  input buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         INIT3      !  initialize I/O API
        INTEGER         INDEX1     !  look up names in name tables
        INTEGER         JSTEP3     !  compute time step record numbers
        LOGICAL         RDDICT3    !  read records for    DCTNRY3  files
        LOGICAL         RDBUF3     !  read time steps for BUFFERED files
        LOGICAL         RDCUSTOM   !  read time steps for CUSTOM3  files
        LOGICAL         RDGRDDED   !  read time steps for GRDDED3  files
        LOGICAL         RDBNDARY   !  read time steps for BNDARY3  files
        LOGICAL         RDIDDATA   !  read time steps for IDDATA3  files
        LOGICAL         RDPROFIL   !  read time steps for PROFIL3  files
        LOGICAL         RDGRNEST   !  read time steps for GRNEST3  files
        LOGICAL         RDSMATRX   !  read time steps for SMATRX33  files

        EXTERNAL        INIT3, INDEX1, JSTEP3, RDDICT3, RDBUF3,
     &                  RDGRDDED, RDBNDARY, RDIDDATA, RDGRNEST, RDSMATRX
        EXTERNAL        INITBLK3   !  block data: initialize I/O state


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         FID             !  subscript  for STATE3 arrays
        INTEGER         VID, VAR, V     !  subscripts for STATE3 arrays
        INTEGER         STEP            !  time step record number
        INTEGER         STEP2           !  mod( step, 2) if tstep < 0
        INTEGER         IERR            !  netCDF error status return
        INTEGER         DIMT( 5 )       !  corner   for NCVGT()
        INTEGER         DELT( 5 )       !  diagonal for NCVGT()
        INTEGER         FLAGS( 2,MXVARS3 )!  values array from NCVGT()
        INTEGER         FLAG1, FLAG2    !  test values for FLAGS
        CHARACTER*16    FIL16           !  scratch file-name buffer
        CHARACTER*16    VAR16           !  scratch vble-name buffer
        CHARACTER*256   MESG
        LOGICAL         EFLAG

!DEC$ IF DEFINED (_WIN32)
!DEC$ ELSE
#ifdef IOAPICPL
        INTEGER       COUNT, SKIP, TYPE
        LOGICAL       READ3V
        EXTERNAL      READ3V
#endif
!DEC$ ENDIF

C***********************************************************************
C   begin body of function  READ3
C.......   Check that Models-3 I/O has been initialized:
 
        EFLAG = .FALSE.
!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV = INIT3()
            EFLAG = .TRUE.
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )
        IF ( EFLAG ) THEN
            READ3 = .FALSE.
            CALL M3MSG2(  'READ3:  I/O API not yet initialized.' )
            RETURN
        END IF

C...........   Check length of name arguments; copy into length=16 buffers
            
        IF ( LEN( FNAME ) .GT. 16 ) THEN
            MESG = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max file name length 16; actual:', LEN( FNAME )
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
            READ3 = .FALSE.
            RETURN
        ELSE IF ( LEN( VNAME ) .GT. 16 ) THEN
            MESG = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max vble name length 16; actual:', LEN( VNAME )
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
	    READ3 = .FALSE.
            RETURN
        END IF          !  if len( fname ) > 16, or if len( vname ) > 16

        FIL16 = FNAME
        VAR16 = VNAME
        

C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN  !  file not available

            MESG = 'File:  '//FIL16// ' not yet opened.'
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
            READ3 = .FALSE.
            RETURN

        ELSE IF ( VOLAT3( FID ) ) THEN     !  volatile file:  synch with disk

!$OMP CRITICAL( S_NC )
            CALL NCSNC( CDFID3( FID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) THEN

                WRITE( MESG,91010 )
     &              'netCDF error number', IERR,
     &              'Error with disk synchronization for file:  '
     &              // FIL16
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN

            END IF      !  if ncsnc() failed

        END IF          !  if file not available, or if file is volatile


C.......   Check availability of requested  layer, variable:

        VAR16 = VNAME   !  fixed-length-16 scratch copy of name

        IF ( FTYPE3( FID ) .EQ. DCTNRY3 ) THEN

            READ3 = RDDICT3 ( FID, VAR16 )
            RETURN
        
        ELSE IF ( VAR16 .EQ. ALLVAR3 ) THEN
            
            VID = ALLAYS3
        
        ELSE 
            
            VID = INDEX1 ( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )
            
            IF ( VID .EQ. 0 ) THEN
                 MESG = 'File ' // FLIST3( FID )//' contains variables'
                 CALL M3MSG2( MESG )
                 DO  VAR=1,NVARS3( FID ), 3
                    WRITE( MESG,91040 )
     &              (VLIST3( V,FID ),V=VAR,MIN( VAR+2, NVARS3( FID ) ) )
                    CALL M3MSG2( MESG )
                 END DO
                 MESG = 'Requested variable "' //  VAR16 // 
     &                  '" not available'
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN
            END IF

        END IF          !  end check on VNAME

        IF ( ( LAYER .NE. ALLAYS3 ) 
     &      .AND. ( LAYER .LT. 1  .OR.  
     &              LAYER .GT. NLAYS3( FID ) ) ) THEN

            WRITE( MESG,91010 ) 'Requested layer:', LAYER
            CALL M3MSG2( MESG )
	    WRITE( MESG,91010 )
     &          'Layers in the file:  1 ...', NLAYS3( FID )
            CALL M3MSG2( MESG )
            MESG = 'LAYER not available in file ' // FIL16
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
            READ3 = .FALSE.
            RETURN

        END IF          !  end check on layer-number

                
C.......   If buffered virtual file, use RDBUF3():

        IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN     !  BUFFERED "file"

            READ3 = RDBUF3( FID, VID, LAYER, JDATE, JTIME, BUFFER )
            RETURN
        
        END IF  
        
C...........   Compute record number, and check availability:

        STEP = JSTEP3( JDATE, JTIME,
     &                 SDATE3( FID ),
     &                 STIME3( FID ),
     &                 TSTEP3( FID ) )
        
        IF ( STEP .LT. 0 ) THEN

            MESG = 'Time step error reading file:  ' // FIL16
            CALL M3MSG2( MESG )
            WRITE( MESG,91020 )
     &          'Requested date & time:    ', JDATE, JTIME
            CALL M3MSG2( MESG )
            WRITE( MESG,91020 )
     &          'File starting date & time:',
     &          SDATE3( FID ), STIME3( FID )
            CALL M3MSG2( MESG )
            WRITE( MESG,91030 )
     &          'File time step:           ', TSTEP3( FID )
            CALL M3MSG2( MESG )
            MESG = 'Time step error reading file:  ' // FIL16
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
            READ3 = .FALSE.
            RETURN
            
        END IF          !  check on step number

        IF ( TSTEP3( FID ) .LT. 0 ) THEN
            STEP2 = 1 + MOD( STEP - 1, 2 )
            FLAG1 = JDATE
            FLAG2 = JTIME
        ELSE IF ( TSTEP3( FID ) .GT. 0 ) THEN
            STEP2 = STEP
            FLAG1 = JDATE
            FLAG2 = JTIME
        ELSE    ! tstep3( fid ) = 0
            STEP2 = STEP
            FLAG1 = 0
            FLAG2 = 0
        END IF
              

!DEC$ IF DEFINED (_WIN32)
!DEC$ ELSE
#ifdef IOAPICPL
        IF ( CDFID3( FID ) .EQ. VIRFIL3 ) THEN     !  virtual "file"
           IF ( LAYER .EQ. ALLAYS3 ) THEN
              COUNT = BSIZE3(FID)*NLAYS3(FID)
              SKIP = 0
           ELSE
              COUNT = BSIZE3(FID)
              SKIP = (LAYER - 1) * BSIZE3(FID)
           ENDIF
           IF ( VAR16 .EQ. ALLVAR3 ) THEN
              EFLAG = .TRUE.
              DO VID=1,NVARS3(FID)
                 TYPE = VTYPE3(VID,FID)
                 EFLAG = EFLAG .AND.
     $                READ3V(PLIST3(FID),VLIST3(VID,FID),SKIP,
     $                       FLAG1,FLAG2,
     $                       BUFFER(1+(VID-1)*BSIZE3(FID)*NLAYS3(FID)),
     $                       COUNT,TYPE)
              ENDDO
              READ3 = EFLAG
           ELSE
              TYPE = VTYPE3(VID,FID)
              READ3 = READ3V(PLIST3(FID),VNAME,SKIP,
     $                       FLAG1,FLAG2,BUFFER,
     $                       COUNT,TYPE)
           ENDIF

           RETURN
        
        END IF  
#endif
!DEC$ ENDIF

        DIMT( 1 ) = 1           !  field:  date or time
        DELT( 1 ) = 2           !  extent:  entire field
        DIMT( 3 ) = STEP2       !  timestep dimension
        DELT( 3 ) = 1           !  extent in timestep dimension
        
        IF ( VID .GT. 0 ) THEN  !  reading just one variable

            DIMT( 2 ) = VID     !  variable-number
            DELT( 2 ) = 1       !  extent:  one variable
        
!$OMP CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS( 1,VID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written
               
                READ3 = .FALSE.
                RETURN
                        
            ELSE IF ( IERR .NE. 0 ) THEN
               
                WRITE( MESG,91010 ) 'netCDF error number', IERR
                CALL M3MSG2( MESG )
                MESG = 'Error reading netCDF time step flag for ' //
     &                 VLIST3( VID,FID ) // ' from ' // FIL16
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN
               
            ELSE IF( FLAGS( 1,VID ) .NE. FLAG1 .OR.
     &               FLAGS( 2,VID ) .NE. FLAG2 ) THEN
               
                WRITE( MESG,91020 )
     &              'Requested date & time:', JDATE, JTIME
                CALL M3MSG2( MESG )
                MESG = 'Variable ' // VLIST3( VID,FID ) // 
     &                 ' not available in file ' // FIL16
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN
               
            END IF          !  if ierr bad or if timestep flags bad
                   
        ELSE            !  reading all variables
        
            DIMT( 2 ) = 1               !  initial variable-number
            DELT( 2 ) = MAX( 1, NVARS3( FID ) )   !  extent:  all variables
        
!$OMP CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP END CRITICAL( S_NC )
        
            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written
               
                    WRITE( MESG,91020 )
     &              'Reading ALL variables -- requested date & time:', 
     &              JDATE, JTIME
                    CALL M3MSG2( MESG )
                    MESG = 'Time step not yet written in file ' // FIL16
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN
                        
            ELSE IF ( IERR .NE. 0 ) THEN
               
                WRITE( MESG,91010 ) 'netCDF error number', IERR
                CALL M3MSG2( MESG )
                MESG =
     &          'Error reading netCDF time step flags for file:  '
     &          // FIL16
                CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                READ3 = .FALSE.
                RETURN
               
            END IF          !  if ierr nonzero or not for NCVGT1()
            
            
C...........   Check time step flags for all variables:
        
            IF ( NVARS3( FID ) .GT. 0 ) THEN

                DO  22  VAR = 1, MAX( NVARS3( FID ), 1 )

                    IF ( FLAGS( 1,VAR ) .NE. FLAG1  .OR.
     &                   FLAGS( 2,VAR ) .NE. FLAG2  ) THEN

                        WRITE( MESG,91020 )
     &                  'Reading ALL variables -- ' //
     &                  'requested date & time:', 
     &                  JDATE, JTIME
                        CALL M3MSG2( MESG )
                        MESG = 'Time step not available in file ' // 
     &                  FIL16 // ' for variable ' // VLIST3( VAR, FID )
                        CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                        READ3 = .FALSE.
                        RETURN

                    END IF          !  if bad flag value

22              CONTINUE	!  end loop on user-variables VAR

            ELSE	!  nvars zero (structured no-user-vble time step)

                IF ( FLAGS( 1,1 ) .NE. FLAG1  .OR.
     &               FLAGS( 2,1 ) .NE. FLAG2  ) THEN
           
                    WRITE( MESG,91020 )
     &              'Reading entire time step -- ' //
     &              'requested date & time:', JDATE, JTIME
                    CALL M3MSG2( MESG )
                    MESG = 'Time step not available in file ' // FIL16
                    CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
                    READ3 = .FALSE.
                    RETURN
           
                END IF          !  if bad flag value

            END IF	!  if nvars positive, or not

        END IF          !  if reading one variable or many (checking flags)
        

C...........   Read data from file into BUFFER()

        IF ( FTYPE3( FID ) .EQ. CUSTOM3 ) THEN

            READ3 = RDCUSTOM( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. GRDDED3 ) THEN

            READ3 = RDGRDDED( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. BNDARY3 ) THEN

            READ3 = RDBNDARY( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. IDDATA3 ) THEN

            READ3 = RDIDDATA( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. PROFIL3 ) THEN

            READ3 = RDPROFIL( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. GRNEST3 ) THEN

            READ3 = RDGRNEST( FID, VID, LAYER, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. SMATRX3 ) THEN

            READ3 = RDSMATRX( FID, VID, STEP, BUFFER )

        ELSE IF ( FTYPE3( FID ) .EQ. TSRIES3 ) THEN
 
            READ3 = RDGRDDED( FID, VID, LAYER, STEP, BUFFER )
 
        ELSE IF ( FTYPE3( FID ) .EQ. PTRFLY3 ) THEN

            READ3 = RDGRDDED( FID, VID, LAYER, STEP, BUFFER )

        ELSE    !  illegal file type

            WRITE( MESG,91010 )
     &      'File:  ' // FIL16 // ' of unknown type:', FTYPE3( FID )
            CALL M3WARN( 'READ3', JDATE, JTIME, MESG )
            READ3 = .FALSE.

        END IF          !  if file type dictionary, custom, gridded, ...


        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91000   FORMAT ( 5A )

91010   FORMAT ( 3 ( A , :, I5, :, 2X ) )

91020   FORMAT ( A , I9, ':' , I6.6, :, A )

91030   FORMAT ( A , I6.6 )

91040   FORMAT ( 3 ( '"', A16, '"', :, 4X ) )

        END

