
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................


        PROGRAM  M3WNDW

C***********************************************************************
C  program body starts at line  84
C
C  FUNCTION:
C       Window a subrectangle of the grid from gridded input file
C       specified time period, and write it to the output file.
C
C  PRECONDITIONS REQUIRED:
C       Machine with stack-allocated AUTO local variables (e.g., CRAY)
C       consistency with FORIO:PARMS3.EXT for name and description lengths.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       GETNUM, NEXTIME, STATSTEP, Models-3 I/O.
C
C  REVISION  HISTORY:
C      Prototype 5/96 by CJC
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'  !  I/O parameter definitions
        INCLUDE 'FDESC3.EXT'  !  file header data structures
        INCLUDE 'IODECL3.EXT' !  I/O definitions and declarations


C...........   EXTERNAL FUNCTIONS and their descriptions:

        LOGICAL       GETYN
        CHARACTER*16  PROMPTMFILE
        INTEGER       GETNUM, IARGC, INDEX1, SECSDIFF, SEC2TIME, 
     &                TIME2SEC, TRIMLEN
        REAL          GETREAL

        EXTERNAL  GETNUM,   GETREAL,  GETYN,    INDEX1, PROMPTMFILE,
     &            SECSDIFF, SEC2TIME, TIME2SEC, TRIMLEN


C...........   PARAMETERS and their descriptions:

        CHARACTER*16	BLANK16
        PARAMETER     ( BLANK16 = ' ' )


C...........   LOCAL VARIABLES and their descriptions:

        INTEGER         LOGDEV  !  unit number for log file
        INTEGER         ARGCNT  !  number of command-line args, from IARGC()
        CHARACTER*256   ENVBUF  !  value from command line arguments
        CHARACTER*256   MESG    !  message buffer for M3EXIT(), etc.

        CHARACTER*16    INAME   !  logical name of the  input file
        CHARACTER*16    WNAME   !  logical name of the output file

        INTEGER         NCOLS   ! grid dimensions, from INAME header
        INTEGER         NROWS   ! grid dimensions, from INAME header
        INTEGER         NLAYS   ! grid dimensions, from INAME header
        INTEGER         NVARS   !  number of vbles in WNAME
        INTEGER         SDATE   !  starting date, from user
        INTEGER         STIME   !  starting time, from user
        INTEGER         JDATE   !  current date
        INTEGER         JTIME   !  current time
        INTEGER         TSTEP   !  time step, from INAME header
        INTEGER         RUNLEN  !  duration, HHMMSS from user
        INTEGER         NSTEPS  !  duration in TSTEPs
        INTEGER         I       !  scratch variables
        INTEGER         LOCOL   !  window boundary 
        INTEGER         HICOL   !  window boundary 
        INTEGER         LOROW   !  window boundary 
        INTEGER         HIROW   !  window boundary 

C.........................................................................
C   begin body of program  M3WNDW

        WRITE ( *,92000 )
     &  ' ',
     &  'Program M3WNDW to window variables to a specified subgrid',
     &  'from a GRIDDED, Models-3 file for a specified time period',
     &  'write them to another such file.', ' ',
     &  'You need to have set environment variables for the input',
     &  'and output file logical names.  You will be asked to select',
     &  'the window into the grid and time period to be windowed.',
     &  ' ',
     &  'USAGE:  m3wndw [INFILE OUTFILE]',
     &  '(and then answer the prompts). ',
     &  ' '

        LOGDEV = INIT3()
        ARGCNT = IARGC()
 
        IF ( ARGCNT .EQ. 0 ) THEN       !  get names from user

            INAME = PROMPTMFILE( 'Enter logical name for  INPUT FILE',
     &                           FSREAD3, 'INFILE', 'M3WNDW' )

        ELSE IF ( ARGCNT .EQ. 2 ) THEN

            CALL GETARG( 1, ENVBUF )
            INAME = ENVBUF( 1:16 )
            IF ( .NOT. OPEN3( INAME, FSREAD3, 'M3WNDW' ) ) THEN
                CALL M3EXIT( 'M3WNDW', 0, 0,
     &                       'Could not open input file "' 
     &                       // INAME( 1:TRIMLEN( INAME ) ) // '"', 
     &                       3 )
            END IF

            CALL GETARG( 2, ENVBUF )
            WNAME = ENVBUF( 1:16 )

        ELSE

            CALL M3EXIT( 'M3WNDW', 0, 0, 
     &                   'usage:  m3wndw [INFILE OUTFILE]', 2 )

        END IF

        IF ( .NOT. DESC3( INAME ) ) THEN
            CALL M3EXIT( 'M3WNDW', 0, 0,
     &                  'Could not get description of input file '
     &                  // INAME,
     &                  2 )
        END IF

        IF ( FTYPE3D .NE. GRDDED3 ) THEN
            
            WRITE( MESG, 94010 )
     &          'Input file "' //
     &          INAME( 1:TRIMLEN( INAME ) ) //
     &          '" has type', FTYPE3D, 
     &          '(type GRDDED3==1 required)'
            CALL M3EXIT( 'M3WNDW', 0, 0, MESG, 3 )

        END IF

        NCOLS  = NCOLS3D
        NROWS  = NROWS3D
        NLAYS  = NLAYS3D
        NVARS  = NVARS3D
        SDATE  = SDATE3D
        STIME  = STIME3D
        TSTEP  = TSTEP3D


C.......   Get starting date and time, and duration:

        IF ( TSTEP .EQ. 0 ) THEN        !  time-independent file

            SDATE  = 0
            STIME  = 0
            NSTEPS = 1

        ELSE                            !  time-dependent file

            SDATE  = GETNUM( SDATE3D, 9999999, SDATE3D,
     &                  'Enter starting date (YYYYDDD) for run' )
            STIME  = GETNUM( 0, 239999, STIME3D,
     &                  'Enter starting time (HHMMSS) for run' )
            RUNLEN = SEC2TIME( MXREC3D * TIME2SEC( TSTEP3D ) )
            RUNLEN = GETNUM( 0, 999999999, RUNLEN,
     &                  'Enter duration (HHMMSS) for run' )
            NSTEPS = TIME2SEC( TSTEP )
            NSTEPS = ( TIME2SEC( RUNLEN ) + NSTEPS - 1 ) / NSTEPS

        END IF          !  time-independent file, or not


C.......   Build description for the output file, and create accordingly:
C.......   Re-use most of the input-file description.

        SDATE3D = SDATE
        STIME3D = STIME

        WRITE ( MESG,94010 )
     &  'Input file "' // INAME( 1 : TRIMLEN( INAME ) ) // 
     &  '" has grid "' // GDNAM3D( 1 : TRIMLEN( GDNAM3D ) ) // 
     &  '" with', NCOLS3D, 'cols and', NROWS3D, 'rows.'
        WRITE ( *,92000 )
     &  ' ',
     &  MESG( 1 : TRIMLEN( MESG ) ),
     &  'Now enter the window specifications.  These will be of the',
     &  'form GRID-NAME, LOCOL, HICOL, LOROW, HIROW relative to the',
     &  'input file grid, where',
     &  ' ',
     &  '        LOCOL <= col <= HICOL',
     &  '        LOROW <= row <= HIROW',
     &  ' '
        MESG = 'WNDW_' // GDNAM3D
        CALL GETSTR( 'Enter name for windowed grid', 
     &               MESG( 1:16 ),  GDNAM3D )
        LOCOL   = GETNUM( 1,     NCOLS3D, 1, 'Enter LOCOL' )
        HICOL   = GETNUM( LOCOL, NCOLS3D, 1, 'Enter HICOL' )
        LOROW   = GETNUM( 1,     NROWS3D, 1, 'Enter LOROW' )
        HIROW   = GETNUM( LOROW, NROWS3D, 1, 'Enter HIROW' )

        XORIG3D = XORIG3D + DBLE( LOCOL - 1 ) * XCELL3D
        YORIG3D = YORIG3D + DBLE( LOROW - 1 ) * YCELL3D
        NCOLS3D = HICOL - LOCOL + 1
        NROWS3D = HIROW - LOROW + 1

        IF ( ARGCNT .EQ. 0 ) THEN
            WNAME = PROMPTMFILE( 'Enter logical name for OUTPUT FILE',
     &                           FSUNKN3, 'OUTFILE', 'M3WNDW' )
        ELSE    !  argcnt = 2:
            IF ( .NOT. OPEN3( WNAME, FSUNKN3, 'M3WNDW' ) ) THEN
                CALL M3ERR( 'M3WNDW', SDATE, STIME,
     &                      'Could not open output file ' // WNAME,
     &                      .TRUE. )
            END IF
        END IF          !  if argcnt zero, or 2


C.......   Process this period in the input file:

        JDATE = SDATE
        JTIME = STIME

        DO  322  I = 1, NSTEPS

            CALL WNDWSTEP( NCOLS, NROWS, NLAYS, NVARS,
     &                     LOCOL, HICOL, LOROW, HIROW,
     &                     JDATE, JTIME,
     &                     INAME, WNAME, LOGDEV )

            CALL NEXTIME( JDATE, JTIME, TSTEP )

322     CONTINUE        !  end loop on time steps


        CALL M3EXIT( 'M3WNDW', 0, 0,
     &               'Program  M3WNDW  completed successfully', 0 )


C..........................   FORMAT STATEMENTS  ....................
C...........   Informational (LOG) message formats... 92xxx

92000   FORMAT ( 5X , A )


C...........   Internal bufferring formats  ......... 94xxx 


94010	FORMAT ( 10 ( A, :, I7, :, 2X ) )

        END

